/*
 * Copyright (c) y2k Micha Moskal <malekith@topnet.pl>
 *
 * This file is part of libcftp - cftp protocol implementation.
 * CryptoFTP is free software. You can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License (see COPYING.LGPL 
 * for details).
 *
 * $Id: random.c,v 1.3 2000/05/04 19:09:35 malekith Exp $
 */

#include "config.h"
#include "sha.h"
#include "utils.h"

#if defined(__unix__) || defined(__CYGWIN__)
#include <unistd.h>
#endif

/* have to be (POOL_SIZE%20) == 0 */
#define POOL_SIZE 500

struct entropy_pool {
	int locked;
	byte pool[POOL_SIZE + 20];
	int index;
	struct sha_ctx sha;
	double last_time;
};

static struct entropy_pool oep;
static struct entropy_pool *ep = &oep;

static void lock(void)
{
	while (ep->locked) { usleep(100000); }
	ep->locked = 1;
}

static void unlock(void)
{
	ep->locked = 0;
}

static void round(void)
{
	register int n;
	register byte *src, *dst;
	
	sha_init(&ep->sha);
	sha_update(&ep->sha, ep->pool, POOL_SIZE + 20);
	sha_final(&ep->sha);
	sha_digest(&ep->sha, ep->pool + POOL_SIZE);
	
	n = 20;
	src = ep->pool + POOL_SIZE;
	dst = ep->pool + ep->index;
	while (n--)
		*dst++ ^= *src++;
	ep->index += 20;
	ep->index %= POOL_SIZE;
}

void random_add(const void *buf, int len)
{
	lock();
	for (; len > 0; buf = (byte*)buf + 20, len -= 20) {
		memcpy(ep->pool + POOL_SIZE, buf, len < 20 ? len : 20);
		round();
	}
	unlock();
}

void random_get(void *_buf, int len)
{
	byte *buf = _buf;
	lock();
	for (; len > 0; buf += 20, len -= 20) {
		round();
		memcpy(buf, ep->pool + POOL_SIZE, len < 20 ? len : 20);
	}
	unlock();
}

int random_byte(void)
{
	byte r;
	random_get(&r, 1);
	return r;
}

int rnd(void)
{
	return random_byte();
}

u32 random_int(void)
{
	u32 r;
	random_get(&r, 4);
	return r;
}


void random_mark(void)
{
	double d[2];
	d[0] = dtime();
	d[1] = dtime() - ep->last_time;
	ep->last_time = dtime();
	random_add(d,sizeof(d));
}

