/*
 * Copyright (c) y2k Micha Moskal <malekith@topnet.pl>
 *
 * This file is part of libcftp - cftp protocol implementation.
 * CryptoFTP is free software. You can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License (see COPYING.LGPL 
 * for details).
 *
 * $Id: cipher.h,v 1.1.1.1 2000/04/22 10:15:53 malekith Exp $
 */

struct cipher;
typedef void (*cipher_func)(struct cipher *self, void *data, int len);
typedef void (*init_cipher_func)(struct cipher *self, const void *key, int len);

struct cipher {
		/* set key */
	init_cipher_func init;
		/* do the dirty work */
	cipher_func cipher, decipher;
		/* frees given cipher struct, zeroing the mem first */
	void (*free)(struct cipher *self);
		/* maximal significant keylen for (*init) func [bytes] */
	int keylen; 
		/* length_of_data_passed_to_[de]cipher % blocklen == 0 */
	int blocklen; 
		/* short id */
	const char *name; 
		/* if non-NULL should be sent back to client */
	const char *disabled; 
};

struct cipher2 {
	struct cipher *(*creator)(void);
	const char *name;
	const char *disabled;
	struct cipher *e, *d;
};

/* null terminated table of supported ciphers */
extern struct cipher2 ciphers[]; 

void init_ciphers(void);
void free_cipher(void);
struct cipher2 *find_cipher(const char *name);

