/*
 * Copyright (c) y2k Micha Moskal <malekith@topnet.pl>
 *
 * This file is part of libcftp - cftp protocol implementation.
 * CryptoFTP is free software. You can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License (see COPYING.LGPL 
 * for details).
 *
 * $Id: clib_int.h,v 1.3 2000/05/03 01:04:50 malekith Exp $
 */

#include "cftp.h"
#include "cipher.h"
#include "utils.h"
#include "protocol.h"

#ifdef WINDA
#include <winsock.h>
#else
#include <arpa/inet.h>
#endif

#ifdef USE_ZLIB
#include "zlib.h"

struct zd_state {
	z_stream s;
	byte *pending;
	int pending_size;
	int initialised;
	int csize;
};
#endif /* USE_ZLIB */

struct zc_state {
#ifdef USE_ZLIB	
	z_stream s;
	int compress;
#endif /* USE_ZLIB */
	byte buf[CHUNK_SIZE];
	int chunk_pos;
	int pktid;
};

#define CTX struct clib_state

struct clib_int {
	struct cipher *e, *d;
	struct cipher2 *c2;
	int ukl;
	int add_nl;
	char *key, *hostline;
	int encrypting;
	struct zc_state zc;
#ifdef USE_ZLIB	
	struct zd_state zd;
#endif /* USE_ZLIB */

	/* ls status */
	int ls_in_progress;
	int ls_last;
	struct pkt_response *ls_resp;
	byte *ls_ptr;

	/* get status */
	int get_in_progress;
	int get_aborting;
	
	/* */
	SOCKET fwd_local;
	byte target_addr[16];
	int v6;
	CTX *fwd_ctx;
	
	int dead;
};


/* common.c */
int set_error(CTX *ctx, char *str);
int set_sock_error(CTX *ctx, const char *string);
int read_l(SOCKET fd, void *ptr, int size);
int write_l(SOCKET fd, const void *ptr, int size);
int send_packet(CTX *ctx, int id, void *data, int len);
struct packet *recv_packet(CTX *ctx);
void free_packet(struct packet *p);
int check_in(CTX *ctx);
int z_decompress_size(CTX *ctx);
int version_agreement(CTX * ctx);
struct packet *recv_packet_x(CTX *ctx, int id);
int proto_err(CTX *ctx, const char *msg);

/* these will compress or not -- uniform iface no matter #ifdef USE_ZLIB */
int z_compress(CTX *ctx, const void *buf, int len);
int z_finish_compress(CTX *ctx);
int z_start_compress(CTX *ctx, int pktid, int compress);

/* clib.c */
extern char *home_dir;

/* clib_utils.c */
char *xprintf(const char *fmt, ... );

