/*
 * Copyright (c) y2k Micha Moskal <malekith@topnet.pl>
 *
 * This file is part of libcftp - cftp protocol implementation.
 * CryptoFTP is free software. You can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License (see COPYING.LGPL 
 * for details).
 *
 * $Id: mpz.c,v 1.2 2000/04/23 16:39:03 malekith Exp $
 */

#include "config.h"
#include "gmp.h"
#include "utils.h"
#include "prime_table.h"

#include <stdio.h>

void xmpz_to_byte(mpz_t d, byte *buf, int len)
{
	char *p, *r;
	p = r = mpz_get_str(0, 16, d);

	while (len-- > 0 && *p) {
		if (*p>='A'&&*p<='F') *p |= 0x20;
		if (*p >= '0' && *p <= '9')
			*p -= '0';
		else
			*p -= 'a' - 10;
		*buf = (*p<<4)&0xf0;
		if(!*++p) break;
		if (*p>='A'&&*p<='F') *p |= 0x20;
		if (*p >= '0' && *p <= '9')
			*p -= '0';
		else
			*p -= 'a' - 10;
		*buf++ |= *p & 0x0f;
		p++;
	}

	xfree0(r, strlen(r));
}

void xmpz_from_byte(mpz_t d, byte *buf, int len)
{
	char *p, *r;
	char *hex = "0123456789abcdef";
	p = r = xmalloc(len * 2 + 1);
	while (len-->0) {
		*p++ = hex[(*buf>>4)&0xf];
		*p++ = hex[*buf&0xf];
		buf++;
	}
	*p = 0;
	mpz_init_set_str(d, r, 16);
	xfree0(r, strlen(r));
}

void xmpz_get_rand(mpz_t v, int bits)
{
	byte *buf;
	buf = xmalloc(bits/8);
	random_get(buf, bits/8);
	buf[0] |= 0x80;
	xmpz_from_byte(v, buf, bits/8);
	xfree0(buf, bits/8);
}

/* override default mpz_random() with cryptogrphicly strong one */
void mpz_random(mpz_t rop, mp_size_t max)
{
	xmpz_get_rand(rop, max * sizeof(mp_limb_t) * 8);
}

static unsigned long small_factor(mpz_t n)
{
	int k;
	const unsigned long *p;

	for (k = NUMBER_OF_PRIMES, p = primes; k-- > 0; p++)
		if (mpz_fdiv_ui(n, *p) == 0)
			return *p;
	return 0;
}

extern int rsa_verbose;

void xmpz_get_prime(mpz_t v, int bits)
{
	mpz_t tmp;
	
	mpz_init_set_ui(tmp, 3);
	
	for (;;) {
		xmpz_get_rand(v, bits);
		mpz_ior(v, v, tmp);
		if (small_factor(v) == 0 && mpz_probab_prime_p(v, 25) == 1)
			break;
		if (rsa_verbose)
			fprintf(stderr, ".");
	}
	
	mpz_clear(tmp);
}

void xmpz_get_rel_prime(mpz_t out, mpz_t p, int bits)
{
	mpz_t gcd;
	
	mpz_init(gcd);
	
	do {
		xmpz_get_rand(out, bits);
		mpz_gcd(gcd, out, p);
	} while (mpz_cmp_ui(gcd, 1) != 0);
	mpz_clear(gcd);
}
